@@ -1,9 +1,10 @@
 package Net::DNS::Domain;
 
-use strict;
-use warnings;
-
-our $VERSION = (qw$Id: Domain.pm 1911 2023-04-17 12:30:59Z willem $)[2];
+#
+# $Id: Domain.pm 1406 2015-10-05 08:25:49Z willem $
+#
+use vars qw($VERSION);
+$VERSION = (qw$LastChangedRevision: 1406 $)[1];
 
 
 =head1 NAME
@@ -14,7 +15,7 @@ Net::DNS::Domain - DNS domains
 
     use Net::DNS::Domain;
 
-    $domain = Net::DNS::Domain->new('example.com');
+    $domain = new Net::DNS::Domain('example.com');
     $name   = $domain->name;
 
 =head1 DESCRIPTION
@@ -26,8 +27,8 @@ Each domain object instance represents a single DNS domain which
 has a fixed identity throughout its lifetime.
 
 Internally, the primary representation is a (possibly empty) list
-of ASCII domain name labels, and optional link to an origin domain
-object topologically closer to the DNS root.
+of ASCII domain name labels, and optional link to an arbitrary
+origin domain object topologically closer to the DNS root.
 
 The computational expense of Unicode character-set conversion is
 partially mitigated by use of caches.
@@ -35,22 +36,26 @@ partially mitigated by use of caches.
 =cut
 
 
+use strict;
 use integer;
 use Carp;
 
 
 use constant ASCII => ref eval {
 	require Encode;
-	Encode::find_encoding('ascii');
+	Encode::find_encoding('ascii');				# encoding object
 };
 
-use constant UTF8 => scalar eval {	## not UTF-EBCDIC  [see Unicode TR#16 3.6]
-	Encode::encode_utf8( chr(182) ) eq pack( 'H*', 'C2B6' );
+use constant LIBUTF8 => scalar eval {
+	Encode::decode_utf8( chr(91) ) eq '[';			# not UTF-EBCDIC  [see UTR#16 3.6]
 };
 
-use constant LIBIDN2  => defined eval { require Net::LibIDN2 };
-use constant IDN2FLAG => LIBIDN2 ? &Net::LibIDN2::IDN2_NFC_INPUT + &Net::LibIDN2::IDN2_NONTRANSITIONAL : 0;
-use constant LIBIDN   => LIBIDN2 ? undef : defined eval { require Net::LibIDN };
+use constant UTF8 => ref eval {
+	LIBUTF8 && Encode::find_encoding('utf8');		# encoding object
+};
+
+use constant LIBIDN => UTF8 && defined eval { require Net::LibIDN; };
+
 
 # perlcc: address of encoding objects must be determined at runtime
 my $ascii = ASCII ? Encode::find_encoding('ascii') : undef;	# Osborn's Law:
@@ -61,7 +66,7 @@ my $utf8  = UTF8  ? Encode::find_encoding('utf8')  : undef;	# Variables won't; c
 
 =head2 new
 
-    $object = Net::DNS::Domain->new('example.com');
+    $object = new Net::DNS::Domain('example.com');
 
 Creates a domain object which represents the DNS domain specified
 by the character string argument. The argument consists of a
@@ -82,22 +87,17 @@ by the Perl compiler.
 The character string presentation format follows the conventions
 for zone files described in RFC1035.
 
-Users should be aware that non-ASCII domain names will be transcoded
-to NFC before encoding, which is an irreversible process.
-
 =cut
 
-my ( %escape, %unescape );		## precalculated ASCII escape tables
-
-our $ORIGIN;
+use vars qw($ORIGIN);
 my ( $cache1, $cache2, $limit ) = ( {}, {}, 100 );
 
 sub new {
 	my ( $class, $s ) = @_;
 	croak 'domain identifier undefined' unless defined $s;
 
-	my $index = join '', $s, $class, $ORIGIN || '';		# cache key
-	my $cache = $$cache1{$index} ||= $$cache2{$index};	# two layer cache
+	my $k = join '', $s, $class, $ORIGIN || '';		# cache key
+	my $cache = $$cache1{$k} ||= $$cache2{$k};		# two layer cache
 	return $cache if defined $cache;
 
 	( $cache1, $cache2, $limit ) = ( {}, $cache1, 500 ) unless $limit--;	# recycle cache
@@ -107,29 +107,15 @@ sub new {
 	$s =~ s/\\\\/\\092/g;					# disguise escaped escape
 	$s =~ s/\\\./\\046/g;					# disguise escaped dot
 
-	my $label = $self->{label} = ( $s eq '@' ) ? [] : [split /\056/, _encode_utf8($s)];
+	my $label = $self->{label} = $s eq '@' ? [] : [split /\056/, _encode_ascii($s)];
 
-	foreach (@$label) {
-		croak qq(empty label in "$s") unless length;
-
-		if ( LIBIDN2 && UTF8 && /[^\000-\177]/ ) {
-			my $rc = 0;
-			$_ = Net::LibIDN2::idn2_to_ascii_8( $_, IDN2FLAG, $rc );
-			croak Net::LibIDN2::idn2_strerror($rc) unless $_;
-		}
-
-		if ( LIBIDN && UTF8 && /[^\000-\177]/ ) {
-			$_ = Net::LibIDN::idn_to_ascii( $_, 'utf-8' );
-			croak 'name contains disallowed character' unless $_;
-		}
-
-		s/\134([\060-\071]{3})/$unescape{$1}/eg;	# restore numeric escapes
-		s/\134([^\134])/$1/g;				# restore character escapes
-		s/\134(\134)/$1/g;				# restore escaped escapes
-		croak qq(label too long in "$s") if length > 63;
+	foreach my $l (@$label) {
+		$l = _unescape($l) if $l =~ /\\/;
+		croak 'empty domain label' unless my $size = length($l);
+		( substr( $l, 63 ) = '', carp 'domain label truncated' ) if $size > 63;
 	}
 
-	$$cache1{$index} = $self;				# cache object reference
+	$$cache1{$k} = $self;					# cache object reference
 
 	return $self if $s =~ /\.$/;				# fully qualified name
 	$self->{origin} = $ORIGIN || return $self;		# dynamically scoped $ORIGIN
@@ -152,33 +138,36 @@ numerical escape sequence.
 
 =cut
 
+my $dot = '.';
+
 sub name {
 	my ($self) = @_;
 
 	return $self->{name} if defined $self->{name};
 	return unless defined wantarray;
 
-	my @label = shift->_wire;
-	return $self->{name} = '.' unless scalar @label;
-
-	for (@label) {
-		s/([^\055\101-\132\141-\172\060-\071])/$escape{$1}/eg;
-	}
-
-	return $self->{name} = _decode_ascii( join chr(46), @label );
+	my $lref = $self->{label};
+	my $head = _decode_ascii( join chr(46), map _escape($_), @$lref );
+	my $tail = $self->{origin} || return $self->{name} = $head || $dot;
+	return $self->{name} = $tail->name unless length $head;
+	my $suffix = $tail->name;
+	return $self->{name} = $suffix eq $dot ? $head : join $dot, $head, $suffix;
 }
 
 
 =head2 fqdn
 
-    $fqdn = $domain->fqdn;
+    @fqdn = $domain->fqdn;
 
 Returns a character string containing the fully qualified domain
 name, including the trailing dot.
 
 =cut
 
-sub fqdn { return $name =~ /[.]$/ ? $name : "$name." }		# append trailing dot
+sub fqdn {
+	my $name = &name;
+	return $name =~ /[$dot]$/o ? $name : $name . $dot;	# append trailing dot
+}
 
 
 =head2 xname
@@ -188,21 +177,15 @@ sub fqdn { return $name =~ /[.]$/ ? $name : "$name." }		# append trailing dot
 Interprets an extended name containing Unicode domain name labels
 encoded as Punycode A-labels.
 
-If decoding is not possible, the ACE encoded name is returned.
+Domain names containing Unicode characters are supported if the
+Net::LibIDN module is installed.
 
 =cut
 
 sub xname {
 	my $name = &name;
 
-	if ( LIBIDN2 && UTF8 && $name =~ /xn--/i ) {
-		my $self = shift;
-		return $self->{xname} if defined $self->{xname};
-		my $u8 = Net::LibIDN2::idn2_to_unicode_88($name);
-		return $self->{xname} = $u8 ? $utf8->decode($u8) : $name;
-	}
-
-	if ( LIBIDN && UTF8 && $name =~ /xn--/i ) {
+	if ( LIBIDN && $name =~ /xn--/ ) {
 		my $self = shift;
 		return $self->{xname} if defined $self->{xname};
 		return $self->{xname} = $utf8->decode( Net::LibIDN::idn_to_unicode $name, 'utf-8' );
@@ -220,12 +203,11 @@ Identifies the domain by means of a list of domain labels.
 =cut
 
 sub label {
-	my @label = shift->_wire;
-	for (@label) {
-		s/([^\055\101-\132\141-\172\060-\071])/$escape{$1}/eg;
-		_decode_ascii($_);
-	}
-	return @label;
+	my $self = shift;
+
+	my @head = map _decode_ascii( _escape($_) ), @{$self->{label}};
+	my $tail = $self->{origin} || return (@head);
+	return ( @head, $tail->label );
 }
 
 
@@ -241,14 +223,17 @@ represented by the appropriate escape sequence.
 
 =cut
 
-sub string { return &fqdn }
+sub string {
+	( my $name = &name ) =~ s/(["'\$();@])/\\$1/;		# escape special char
+	return $name =~ /[$dot]$/o ? $name : $name . $dot;	# append trailing dot
+}
 
 
 =head2 origin
 
-    $create = Net::DNS::Domain->origin( $ORIGIN );
-    $result = &$create( sub{ Net::DNS::RR->new( 'mx MX 10 a' ); } );
-    $expect = Net::DNS::RR->new( "mx.$ORIGIN. MX 10 a.$ORIGIN." );
+    $create = origin Net::DNS::Domain( $ORIGIN );
+    $result = &$create( sub{ new Net::DNS::RR( 'mx MX 10 a' ); } );
+    $expect = new Net::DNS::RR( "mx.$ORIGIN. MX 10 a.$ORIGIN." );
 
 Class method which returns a reference to a subroutine wrapper
 which executes a given constructor in a dynamically scoped context
@@ -260,90 +245,107 @@ my $placebo = sub { my $constructor = shift; &$constructor; };
 
 sub origin {
 	my ( $class, $name ) = @_;
-	my $domain = defined $name ? __PACKAGE__->new($name) : return $placebo;
+	my $domain = defined $name ? new Net::DNS::Domain($name) : return $placebo;
 
 	return sub {						# closure w.r.t. $domain
 		my $constructor = shift;
 		local $ORIGIN = $domain;			# dynamically scoped $ORIGIN
 		&$constructor;
-	}
+			}
 }
 
 
 ########################################
 
-sub _decode_ascii {			## ASCII to perl internal encoding
-	local $_ = shift;
+sub _decode_ascii {			## translate ASCII to perl string
+	my $s = shift;
 
 	# partial transliteration for non-ASCII character encodings
-	tr
+	$s =~ tr
 	[\040-\176\000-\377]
-	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
+	[ !"#$%&'()*+,-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
 
-	my $z = length($_) - length($_);			# pre-5.18 taint workaround
-	return ASCII ? substr( $ascii->decode($_), $z ) : $_;
+	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
+	return ASCII ? pack( "a* x$z", $ascii->decode($s) ) : $s;
 }
 
 
-sub _encode_utf8 {			## perl internal encoding to UTF8
-	local $_ = shift;
+sub _encode_ascii {			## translate perl string to ASCII
+	my $s = shift;
+
+	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
+
+	if ( LIBIDN && $s =~ /[^\000-\177]/ ) {
+		my $xn = Net::LibIDN::idn_to_ascii( $s, 'utf-8' );
+		croak 'invalid name' unless $xn;
+		return pack "a* x$z", $xn;
+	}
 
 	# partial transliteration for non-ASCII character encodings
-	tr
-	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~\000-\377]
+	$s =~ tr
+	[ !"#$%&'()*+,-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~\000-\377]
 	[\040-\176\077] unless ASCII;
 
-	my $z = length($_) - length($_);			# pre-5.18 taint workaround
-	return ASCII ? substr( ( UTF8 ? $utf8 : $ascii )->encode($_), $z ) : $_;
+	return ASCII ? pack( "a* x$z", $ascii->encode($s) ) : $s;
 }
 
 
-sub _wire {
-	my $self = shift;
+my %esc = eval {			## precalculated ASCII escape table
+	my %table;
 
-	my $label  = $self->{label};
-	my $origin = $self->{origin};
-	return ( @$label, $origin ? $origin->_wire : () );
-}
+	foreach ( 33 .. 126 ) {					# ASCII printable
+		$table{pack( 'C', $_ )} = pack 'C', $_;
+	}
 
+	# minimal character escapes
+	foreach ( 46, 92 ) {					# \. \\
+		$table{pack( 'C', $_ )} = pack 'C*', 92, $_;
+	}
 
-%escape = eval {			## precalculated ASCII escape table
-	my %table = map { ( chr($_) => chr($_) ) } ( 0 .. 127 );
-
-	foreach my $n ( 0 .. 32, 34, 92, 127 .. 255 ) {		# \ddd
+	foreach my $n ( 0 .. 32, 127 .. 255 ) {			# \ddd
 		my $codepoint = sprintf( '%03u', $n );
 
-		# transliteration for non-ASCII character encodings
+		# partial transliteration for non-ASCII character encodings
 		$codepoint =~ tr [0-9] [\060-\071];
 
 		$table{pack( 'C', $n )} = pack 'C a3', 92, $codepoint;
 	}
 
-	foreach my $n ( 40, 41, 46, 59 ) {			# character escape
-		$table{chr($n)} = pack( 'C2', 92, $n );
-	}
-
 	return %table;
 };
 
 
-%unescape = eval {			## precalculated numeric escape table
+sub _escape {				## Insert escape sequences in string
+	my $s = shift;
+	$s =~ s/([^\055\101-\132\141-\172\060-\071])/$esc{$1}/eg;
+	return $s;
+}
+
+
+my %unesc = eval {			## precalculated numeric escape table
 	my %table;
 
 	foreach my $n ( 0 .. 255 ) {
 		my $key = sprintf( '%03u', $n );
 
-		# transliteration for non-ASCII character encodings
+		# partial transliteration for non-ASCII character encodings
 		$key =~ tr [0-9] [\060-\071];
 
-		$table{$key} = pack 'C', $n;
+		$table{$key} = pack 'C*', $n, $n == 92 ? ($n) : ();
 	}
-	$table{"\060\071\062"} = pack 'C2', 92, 92;		# escaped escape
 
 	return %table;
 };
 
 
+sub _unescape {				## Remove escape sequences in string
+	my $s = shift;
+	$s =~ s/\134([\060-\071]{3})/$unesc{$1}/eg;		# numeric escape
+	$s =~ s/\134(.)/$1/g;					# character escape
+	return $s;
+}
+
+
 1;
 __END__
 
@@ -362,7 +364,7 @@ compiled into the code.
 
 =head1 COPYRIGHT
 
-Copyright (c)2009-2011,2017 Dick Franks.
+Copyright (c)2009-2011 Dick Franks.
 
 All rights reserved.
 
@@ -371,7 +373,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the original copyright notices appear in all copies and that both
+that the above copyright notice appear in all copies and that both that
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -388,10 +390,8 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl> L<Net::DNS> L<Net::LibIDN2>
-L<RFC1034|https://tools.ietf.org/html/rfc1034>
-L<RFC1035|https://tools.ietf.org/html/rfc1035>
-L<RFC5891|https://tools.ietf.org/html/rfc5891>
+L<perl>, L<Net::LibIDN>, L<Net::DNS>, RFC1034, RFC1035, RFC5891,
+Unicode Technical Report #16
 
 =cut
 
