@@ -1,10 +1,10 @@
 package Net::DNS::Packet;
 
 #
-# $Id: Packet.pm 1381 2015-08-25 07:36:09Z willem $
+# $Id: Packet.pm 1408 2015-10-06 20:35:56Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1381 $)[1];
+$VERSION = (qw$LastChangedRevision: 1408 $)[1];
 
 
 =head1 NAME
@@ -114,6 +114,7 @@ sub decode {
 	my $offset = 0;
 	my $self;
 	eval {
+		local $SIG{__DIE__};
 		die 'corrupt wire-format data' if length($$data) < HEADER_LENGTH;
 
 		# header section
@@ -773,18 +774,18 @@ sub truncate {
 	}
 	$self->header->tc(1) if $tc;				# only set if truncated here
 
-	my @list;
-	my @rrset;
 	my %rrset;
+	my @order;
 	foreach my $item ( grep ref($_) ne ref($sigrr), @{$self->{additional}} ) {
 		my $name  = $item->{owner}->canonical;
 		my $class = $item->{class} || 0;
 		my $key	  = pack 'nna*', $class, $item->{type}, $name;
-		CORE::push @rrset, $key unless $rrset{$key};
+		CORE::push @order, $key unless $rrset{$key};
 		CORE::push @{$rrset{$key}}, $item;
 	}
 
-	foreach my $key (@rrset) {
+	my @list;
+	foreach my $key (@order) {
 		my $component = '';
 		my @item      = @{$rrset{$key}};
 		foreach my $item (@item) {
