@@ -1,10 +1,10 @@
 package Net::DNS::Question;
 
 #
-# $Id: Question.pm 1356 2015-06-11 08:44:12Z willem $
+# $Id: Question.pm 1381 2015-08-25 07:36:09Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1356 $)[1];
+$VERSION = (qw$LastChangedRevision: 1381 $)[1];
 
 
 =head1 NAME
@@ -218,8 +218,9 @@ sub qname {
 sub zname { &qname; }
 
 
-=head2 qtype, ztype
+=head2 qtype, ztype, type
 
+    $qtype = $question->type;
     $qtype = $question->qtype;
     $ztype = $question->ztype;
 
@@ -239,8 +240,9 @@ sub qtype { &type; }
 sub ztype { &type; }
 
 
-=head2 qclass, zclass
+=head2 qclass, zclass, class
 
+    $qclass = $question->class;
     $qclass = $question->qclass;
     $zclass = $question->zclass;
 
