@@ -1,10 +1,10 @@
 package Net::DNS::RR::AAAA;
 
 #
-# $Id: AAAA.pm 1360 2015-06-15 09:58:53Z willem $
+# $Id: AAAA.pm 1388 2015-09-07 12:28:16Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1360 $)[1];
+$VERSION = (qw$LastChangedRevision: 1388 $)[1];
 
 
 use strict;
@@ -20,7 +20,7 @@ Net::DNS::RR::AAAA - DNS AAAA resource record
 use integer;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -28,23 +28,23 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{address};
+	return '' unless defined $self->{address};
 	pack 'a16', $self->{address};
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{address};
-	return $self->address_short;
+	return '' unless defined $self->{address};
+	$self->address_short;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->address(shift);
