@@ -1,13 +1,13 @@
 package Net::DNS::RR::CDS;
 
 #
-# $Id: CDS.pm 1584 2017-07-28 16:15:17Z willem $
+# $Id: CDS.pm 1339 2015-04-16 13:01:17Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1584 $)[1];
+use vars qw($VERSION);
+$VERSION = (qw$LastChangedRevision: 1339 $)[1];
 
 
 use strict;
-use warnings;
 use base qw(Net::DNS::RR::DS);
 
 =head1 NAME
@@ -17,44 +17,6 @@ Net::DNS::RR::CDS - DNS CDS resource record
 =cut
 
 
-use integer;
-
-
-sub _format_rdata {			## format rdata portion of RR string.
-	my $self = shift;
-
-	return $self->SUPER::_format_rdata()
-			if scalar grep $_, @{$self}{qw(keytag algorithm digtype)};
-	return defined $self->{algorithm} ? '0 0 0 0' : '';	# RFC8078 mandated notation
-}
-
-
-sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
-
-	return $self->SUPER::_parse_rdata(@_) if $_[1];
-	die 'invalid RDATA (DS delete)' if "@_" !~ m/^0 0 0/;	 # ignore digest
-	$self->algorithm(0);
-}
-
-
-sub algorithm {
-	my ( $self, $arg ) = @_;
-
-	return $self->SUPER::algorithm($arg) if $arg;
-	return $self->SUPER::algorithm() unless defined $arg;
-	@{$self}{qw(keytag algorithm digtype digestbin)} = ( 0, 0, 0, '' );
-}
-
-
-sub digtype {
-	my ( $self, $arg ) = @_;
-
-	return $self->SUPER::digtype($arg) if $arg;
-	return $self->SUPER::digtype();
-}
-
-
 1;
 __END__
 
@@ -86,7 +48,7 @@ other unpredictable behaviour.
 
 =head1 COPYRIGHT
 
-Copyright (c)2014,2017 Dick Franks
+Copyright (c)2014 Dick Franks
 
 All rights reserved.
 
@@ -114,6 +76,6 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::RR::DS>, RFC7344, RFC8078
+L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::RR::DS>, RFC7344
 
 =cut
