@@ -1,10 +1,10 @@
 package Net::DNS::RR::KX;
 
 #
-# $Id: KX.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: KX.pm 1406 2015-10-05 08:25:49Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+$VERSION = (qw$LastChangedRevision: 1406 $)[1];
 
 
 use strict;
@@ -68,7 +68,7 @@ sub exchange {
 	my $self = shift;
 
 	$self->{exchange} = new Net::DNS::DomainName2535(shift) if scalar @_;
-	$self->{exchange}->name if defined wantarray && $self->{exchange};
+	$self->{exchange}->name if $self->{exchange};
 }
 
 
