@@ -1,10 +1,10 @@
 package Net::DNS::RR::NSEC;
 
 #
-# $Id: NSEC.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: NSEC.pm 1406 2015-10-05 08:25:49Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+$VERSION = (qw$LastChangedRevision: 1406 $)[1];
 
 
 use strict;
@@ -61,7 +61,7 @@ sub nxtdname {
 	my $self = shift;
 
 	$self->{nxtdname} = new Net::DNS::DomainName(shift) if scalar @_;
-	$self->{nxtdname}->name if defined wantarray && $self->{nxtdname};
+	$self->{nxtdname}->name if $self->{nxtdname};
 }
 
 
