@@ -1,10 +1,10 @@
 package Net::DNS::RR::RP;
 
 #
-# $Id: RP.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: RP.pm 1406 2015-10-05 08:25:49Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+$VERSION = (qw$LastChangedRevision: 1406 $)[1];
 
 
 use strict;
@@ -62,7 +62,7 @@ sub mbox {
 	my $self = shift;
 
 	$self->{mbox} = new Net::DNS::Mailbox2535(shift) if scalar @_;
-	$self->{mbox}->address if defined wantarray && $self->{mbox};
+	$self->{mbox}->address if $self->{mbox};
 }
 
 
@@ -70,7 +70,7 @@ sub txtdname {
 	my $self = shift;
 
 	$self->{txtdname} = new Net::DNS::DomainName2535(shift) if scalar @_;
-	$self->{txtdname}->name if defined wantarray && $self->{txtdname};
+	$self->{txtdname}->name if $self->{txtdname};
 }
 
 
