@@ -15,10 +15,10 @@ sub UNITCHECK {				## restore %SIG after compilation
 package Net::DNS::RR::SIG;
 
 #
-# $Id: SIG.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: SIG.pm 1425 2015-11-02 05:58:15Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+$VERSION = (qw$LastChangedRevision: 1425 $)[1];
 
 
 use strict;
@@ -34,9 +34,10 @@ Net::DNS::RR::SIG - DNS SIG resource record
 use integer;
 
 use Carp;
-use MIME::Base64;
 use Time::Local;
 
+eval { require MIME::Base64 };
+
 use Net::DNS::Parameters;
 
 use constant DEBUG => 0;
@@ -45,8 +46,8 @@ use constant UTIL => defined eval { require Scalar::Util; };
 
 use constant PRIVATE => defined eval { require Net::DNS::SEC::Private; };
 
-use constant DSA => scalar eval { require Net::DNS::SEC::DSA; };
-use constant RSA => scalar eval { require Net::DNS::SEC::RSA; };
+use constant DSA => defined eval { require Net::DNS::SEC::DSA; };
+use constant RSA => defined eval { require Net::DNS::SEC::RSA; };
 
 use constant DNSSEC => PRIVATE && ( RSA || DSA );
 
@@ -91,7 +92,7 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my @sig64 = split /\s+/, encode_base64( $self->sigbin );
+	my @sig64 = split /\s+/, MIME::Base64::encode( $self->sigbin );
 	my @rdata = ( map( $self->$_, @field ), $self->{signame}->string, @sig64 );
 }
 
@@ -272,7 +273,7 @@ sub signame {
 	my $self = shift;
 
 	$self->{signame} = new Net::DNS::DomainName2535(shift) if scalar @_;
-	$self->{signame}->name if defined wantarray && $self->{signame};
+	$self->{signame}->name if $self->{signame};
 }
 
 
@@ -450,6 +451,10 @@ my $y1998 = timegm( 0, 0, 0, 1, 0, 1998 );
 my $y2026 = timegm( 0, 0, 0, 1, 0, 2026 );
 my $y2082 = $y2026 << 1;
 my $y2054 = $y2082 - $y1998;
+my $m2026 = int( 0x80000000 - $y2026 );
+my $m2054 = int( 0x80000000 - $y2054 );
+my $t2082 = int( $y2082 & 0x7FFFFFFF );
+my $t2100 = 1960058752;
 
 sub _string2time {			## parse time specification string
 	my $arg = shift;
@@ -471,16 +476,26 @@ sub _string2time {			## parse time specification string
 sub _time2string {			## format time specification string
 	my $arg = shift;
 	croak 'undefined time' unless defined $arg;
-	unless ( $arg < 0 ) {
-		my ( $yy, $mm, @dhms ) = reverse( ( gmtime $arg )[0 .. 5] );
-		return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1900, $mm + 1, @dhms;
-	} elsif ( $arg > $y2082 ) {
-		$arg += 86400 unless $arg < $y2054 + 1456704000;    # expunge 29 Feb 2100
-		my ( $yy, $mm, @dhms ) = reverse( ( gmtime( $arg - $y2054 ) )[0 .. 5] );
-		return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1984, $mm + 1, @dhms;
+	my $ls31 = int( $arg & 0x7FFFFFFF );
+	if ( $arg & 0x80000000 ) {
+
+		if ( $ls31 > $t2082 ) {
+			$ls31 += 86400 unless $ls31 < $t2100;	# expunge 29 Feb 2100
+			my ( $yy, $mm, @dhms ) = reverse( ( gmtime( $ls31 + $m2054 ) )[0 .. 5] );
+			return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1984, $mm + 1, @dhms;
+		}
+
+		my ( $yy, $mm, @dhms ) = reverse( ( gmtime( $ls31 + $m2026 ) )[0 .. 5] );
+		return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1956, $mm + 1, @dhms;
+
+
+	} elsif ( $ls31 > $y2026 ) {
+		my ( $yy, $mm, @dhms ) = reverse( ( gmtime( $ls31 - $y2026 ) )[0 .. 5] );
+		return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1956, $mm + 1, @dhms;
 	}
-	my ( $yy, $mm, @dhms ) = reverse( ( gmtime( $arg - $y2026 ) )[0 .. 5] );
-	return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1956, $mm + 1, @dhms;
+
+	my ( $yy, $mm, @dhms ) = reverse( ( gmtime $ls31 )[0 .. 5] );
+	return sprintf '%d%02d%02d%02d%02d%02d', $yy + 1900, $mm + 1, @dhms;
 }
 
 
