@@ -1,10 +1,10 @@
 package Net::DNS::Resolver;
 
 #
-# $Id: Resolver.pm 1399 2015-09-15 18:50:43Z willem $
+# $Id: Resolver.pm 1425 2015-11-02 05:58:15Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1399 $)[1];
+$VERSION = (qw$LastChangedRevision: 1425 $)[1];
 
 =head1 NAME
 
@@ -15,19 +15,13 @@ Net::DNS::Resolver - DNS resolver class
 
 use strict;
 
-use vars qw(@ISA);
+use constant OS_CONF => "Net::DNS::Resolver::$^O";
+use constant DEFAULT => "Net::DNS::Resolver::UNIX";
 
+use constant CONFIG => scalar eval join ' ', 'require', OS_CONF;
 
-local $SIG{__DIE__};
-local $SIG{__WARN__} = sub { };
+use base ( CONFIG ? OS_CONF : DEFAULT );
 
-for ( $^O, 'UNIX' ) {
-	my @parent = ( join '::', __PACKAGE__, $_ );
-	@ISA = @parent if scalar eval "require @parent";
-	last if @ISA;
-}
-
-die 'failed to load platform specific resolver component' unless @ISA;
 
 1;
 
@@ -110,6 +104,9 @@ Except for F</etc/resolv.conf>, files will only be read if owned by the
 effective userid running the program.  In addition, several environment
 variables may contain configuration information; see L</ENVIRONMENT>.
 
+Note that the domain and searchlist keywords are mutually exclusive.
+If both are present, the resulting behaviour is unspecified.
+
 On Windows systems, an attempt is made to determine the system defaults
 using the registry.  Systems with many dynamically configured network
 interfaces may confuse Net::DNS.
@@ -164,8 +161,6 @@ Domain name suffix to be appended to queries of unqualified names.
 
 =item usevc
 
-=item stayopen
-
 =item igntc
 
 =item defnames
@@ -183,6 +178,7 @@ Domain name suffix to be appended to queries of unqualified names.
 For more information on any of these options, please consult the method
 of the same name.
 
+
 =head2 search
 
     $packet = $resolver->search( 'mailhost' );
@@ -219,6 +215,7 @@ Returns a C<Net::DNS::Packet> object, or C<undef> if no answers were found.
 If you need to examine the response packet, whether it contains
 any answers or not, use the send() method instead.
 
+
 =head2 query
 
     $packet = $resolver->query( 'mailhost' );
@@ -239,10 +236,13 @@ Returns a C<Net::DNS::Packet> object, or C<undef> if no answers were found.
 If you need to examine the response packet, whether it contains
 any answers or not, use the send() method instead.
 
+
 =head2 send
 
     $packet = $resolver->send( $packet );
+
     $packet = $resolver->send( 'mailhost.example.com' );
+    $packet = $resolver->query( '192.0.2.1' );
     $packet = $resolver->send( 'example.com', 'MX' );
     $packet = $resolver->send( 'annotation.example.com', 'TXT', 'HS' );
 
@@ -327,12 +327,6 @@ Gets or sets the nameservers to be queried.
 
 Also see the IPv6 transport notes below
 
-=head2 empty_nameservers
-
-    $resolver->empty_nameservers();
-
-Empties the list of nameservers.
- 
 =head2 print
 
     $resolver->print;
@@ -352,12 +346,6 @@ Returns a string representation of the resolver state.
 
 Gets or sets the resolver search list.
 
-=head2 empty_searchlist
-
-    $resolver->empty_searchlist();
-
-Empties the searchlist.
- 
 =head2 port
 
     print 'sending queries to port ', $resolver->port, "\n";
@@ -383,73 +371,74 @@ The default is 0, meaning any port.
 Gets or sets the source address from which queries are sent.
 Convenient for forcing queries from a specific interface on a
 multi-homed host.
-The default is 0.0.0.0, meaning any local address.
+The default is 0, meaning any local address.
+
 
 =head2 bgsend
 
-    $socket = $resolver->bgsend( $packet ) || die $resolver->errorstring;
+    $handle = $resolver->bgsend( $packet ) || die $resolver->errorstring;
 
-    $socket = $resolver->bgsend( 'mailhost.example.com' );
-    $socket = $resolver->bgsend( 'example.com', 'MX' );
-    $socket = $resolver->bgsend( 'annotation.example.com', 'TXT', 'HS' );
+    $handle = $resolver->bgsend( 'mailhost.example.com' );
+    $handle = $resolver->bgsend( '192.0.2.1' );
+    $handle = $resolver->bgsend( 'example.com', 'MX' );
+    $handle = $resolver->bgsend( 'annotation.example.com', 'TXT', 'HS' );
 
-Performs a background DNS query for the given name, i.e., sends a
-query packet to the first destination in the C<nameservers> list and
-returns immediately without waiting for a response.  The program can
-then perform other tasks while awaiting the response from the nameserver.
+Performs a background DNS query for the given name, returns immediately
+without waiting for a response.  The program can then perform other
+tasks while awaiting the response from the nameserver.
 
 The argument list can be either a C<Net::DNS::Packet> object or a list
 of strings.  The record type and class can be omitted; they default to
 A and IN.  If the name looks like an IP address (IPv4 or IPv6),
 an appropriate PTR query will be performed.
 
-Returns an C<IO::Socket::INET> object or C<undef> on error in which
-case the reason for failure can be found through a call to the
-errorstring method.
+Returns an L<IO::Select> object which serves as a handle which is passed
+to subsequent invocations of the C<bgisready> and C<bgread> methods.
+Errors are indicated by returning C<undef> in which case the
+reason for failure can be found by calling the errorstring method.
 
-The program must determine when the socket is ready for reading and
-call C<bgread> to get the response packet.  Either C<bgisready> or
-C<IO::Select> may be used to find out if the socket is ready.
-
-C<bgsend> does not support persistent sockets.
+The program must determine when the underlying socket is ready for
+reading by calling C<bgisready>, then call C<bgread> to get the response. 
+C<bgread> returns C<undef> if no response is received or timeout occurred. 
 
 B<BEWARE>:
-C<bgsend> does not support the usevc option (TCP) and operates on UDP only.
 Answers may not fit in an UDP packet and might be truncated. Truncated 
 packets will B<not> be retried over TCP automatically and should be handled
 by the caller.
 
+
 =head2 bgread
 
-    $packet = $resolver->bgread($socket);
+    $packet = $resolver->bgread($handle);
+
     if ($packet->header->tc) { 
 	# Retry over TCP (blocking).
     }
-    undef $socket;
 
-Reads the answer from a background query (see L</bgsend>).  The argument
-is an C<IO::Socket> object returned by C<bgsend>.
+Reads the answer from a background query (see L</bgsend>).
+The argument is the handle returned by C<bgsend>.
 
-Returns a C<Net::DNS::Packet> object or C<undef> on error.
+Returns a C<Net::DNS::Packet> object or C<undef> if an error
+was encountered or the transaction timed out.
 
-The programmer should close or destroy the socket object after reading it.
 
 =head2 bgisready
 
-    $socket = $resolver->bgsend( 'foo.example.com' );
-    until ($resolver->bgisready($socket)) {
+    $handle = $resolver->bgsend( 'foo.example.com' );
+
+    until ($resolver->bgisready($handle)) {
 	# do some other processing
     }
-    $packet = $resolver->bgread($socket);
+
+    $packet = $resolver->bgread($handle);
+
     if ($packet->header->tc) { 
 	# Retry over TCP (blocking).
     }
-    $socket = undef;
 
-Determines whether a socket is ready for reading.  The argument is
-an C<IO::Socket> object returned by C<bgsend>.
-
-Returns true if the socket is ready, false if not.
+Returns true when the response is available for reading
+or the transaction timed out.
+The argument is the handle returned by C<bgsend>.
 
 
 =head2 tsig
@@ -586,9 +575,11 @@ in which case the default is true.
     $resolver->persistent_udp(1);
 
 Get or set the persistent UDP setting.
-If true, Net::DNS will keep a single UDP socket open for all queries.
-This is useful if you are using UDP and need to make a lot of queries
-or updates.
+If true, a Net::DNS resolver will use the same UDP socket
+for all queries within each address family.
+
+This avoids the cost of creating and tearing down UDP sockets,
+but also defeats source port randomisation.
 
 =head2 igntc
 
@@ -732,8 +723,8 @@ take values are specified as C<option:value>.
 =head1 IPv6 TRANSPORT
 
 The Net::DNS::Resolver library will enable IPv6 transport if the
-appropriate libraries (Socket6 and IO::Socket::INET6) are available
-and the destination nameserver has at least one IPv6 address.
+appropriate libraries (IO::Socket::INET6 or IO::Socket::IP) are
+available and the destination nameserver has an IPv6 address.
 
 The force_v4(), force_v6() and prefer_v6() methods with a non-zero
 argument may be used to configure transport selection.
@@ -772,11 +763,6 @@ For example, if we wanted to cache lookups:
     }
 
 
-=head1 BUGS
-
-bgsend() does not honour the usevc flag and only uses UDP for transport.
-
-
 =head1 COPYRIGHT
 
 Copyright (c)1997-2000 Michael Fuhr.
@@ -785,7 +771,7 @@ Portions Copyright (c)2002-2004 Chris Reinhardt.
 
 Portions Copyright (c)2005 Olaf M. Kolkman, NLnet Labs.
 
-Portions Copyright (c)2014 Dick Franks.
+Portions Copyright (c)2014,2015 Dick Franks.
 
 All rights reserved.
 
