@@ -1,10 +1,11 @@
 package Net::DNS::Resolver::MSWin32;
 
 #
-# $Id: MSWin32.pm 1381 2015-08-25 07:36:09Z willem $
+# $Id: MSWin32.pm 1406 2015-10-05 08:25:49Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1381 $)[1];
+$VERSION = (qw$LastChangedRevision: 1406 $)[1];
+
 
 =head1 NAME
 
@@ -36,8 +37,8 @@ sub _untaint {
 }
 
 
-sub init {
-	my $defaults = shift->defaults;
+sub _init {
+	my $defaults = shift->_defaults;
 
 	my $debug = 0;
 
@@ -55,8 +56,7 @@ sub init {
 	$defaults->nameservers( _untaint @nameservers );
 
 	my $devolution = 0;
-	my @searchlist = _untaint lc $FIXED_INFO->{DomainName};
-	$defaults->domain(@searchlist);
+	my @searchlist = lc $FIXED_INFO->{DomainName};
 
 	if (WINREG) {
 
@@ -100,7 +100,7 @@ sub init {
 	}
 	$defaults->searchlist( _untaint @list );
 
-	$defaults->read_env;
+	$defaults->_read_env;
 }
 
 
