@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::Recurse;
 
 #
-# $Id: Recurse.pm 1390 2015-09-11 11:42:11Z willem $
+# $Id: Recurse.pm 1422 2015-10-27 16:01:20Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1390 $)[1];
+$VERSION = (qw$LastChangedRevision: 1422 $)[1];
 
 
 =head1 NAME
@@ -56,13 +56,13 @@ or at all, a built-in list of IP addresses is used.
 =cut
 
 my @hints;
-my $root;
+my $root = [];
 
 sub hints {
 	my $self = shift;
 
 	return @hints unless scalar @_;
-	$root  = undef;
+	$root  = [];
 	@hints = @_;
 }
 
@@ -74,8 +74,8 @@ as their counterparts in Net::DNS::Resolver.
 
     $packet = $resolver->send( 'www.example.com.', 'A' );
 
-Server-side recursion is suppressed by clearing the recurse flag
-in the packet and recursive name resolution is performed explicitly.
+Server-side recursion is suppressed by clearing the recurse flag in
+query packets and recursive name resolution is performed explicitly.
 
 The query() and search() methods are inherited from Net::DNS::Resolver
 and invoke send() indirectly.
@@ -86,51 +86,35 @@ sub send {
 	return &Net::DNS::Resolver::Base::send if ref $_[1];	# send Net::DNS::Packet
 
 	my $self = shift;
-	my $res	 = bless {cache => {}, %$self}, ref($self);	# Note: cache discarded after query
+	my $res = bless {persistent => {'.' => $root}, %$self}, ref($self);
 
 	my $question = new Net::DNS::Question(@_);
 	my $original = pop(@_);					# sneaky extra argument needed
 	$original = $question unless ref($original);		# to preserve original request
 
 	my ( $head, @tail ) = $question->{qname}->label;
+	my $domain = lc join( '.', @tail ) || '.';
+	my $nslist = $res->{persistent}->{$domain} ||= [];
 	unless ( defined $head ) {
-		return $root if $root;				# root servers cached indefinitely
+		my $defres = bless {%$res}, qw(Net::DNS::Resolver);
+		my @config = $defres->nameserver( $res->hints );
+		my $packet = $defres->send(qw(. NS));
 
-		my $defres = new Net::DNS::Resolver( retry => 1 );
-		my @config = $defres->nameservers( $res->hints );
-		$defres->nameservers( @config, $res->_hints );	# hints, resolv.conf, _hints
-		$defres->udppacketsize(1280);
-
-		my $packet = $defres->send( '.', 'NS' );
-		my %glue;
-
-		# uncoverable branch false	# depends on external data
-		if ($packet) {
-			my @rr = $packet->answer, $packet->authority;
-			my @auth = grep $_->type eq 'NS', @rr;
-			my %auth = map { lc $_->nsdname => 1 } @auth;
-			my @glue = grep $auth{lc $_->name}, $packet->additional;
-			foreach ( grep $_->can('address'), @glue ) {
-				push @{$glue{lc $_->name}}, $_->address;
-			}
-		}
-
-		$defres->nameservers( $res->_hints );
-		$defres->nameservers( map @$_, values %glue );
+		# uncoverable branch false
+		my @rr = $packet->answer, $packet->authority if $packet;
+		my @ns = map $_->nsdname, grep $_->type eq 'NS', @rr;
+		$defres->udppacketsize(1024);
 		$defres->recurse(0);
-		return $root = $defres->send( '.', 'NS' );	# authoritative server
+		$defres->nameservers( @ns, $res->_hints );	# repeat using authoritative server
+		return $defres->send(qw(. NS));
 	}
 
-	my $domain = lc join( '.', @tail ) || '.';
-	my $nslist = $res->{cache}->{$domain} ||= [];
 	if ( scalar @$nslist ) {
-		$self->_diag(";; using cached nameservers for $domain\n");
+		$self->_diag("using cached nameservers for $domain");
 	} else {
 		$domain = lc $question->qname if $question->qtype ne 'NULL';
 		my $packet = $res->send( $domain, 'NULL', 'IN', $original );
-
-		# uncoverable branch true	# depends on external data
-		return unless $packet;
+		return unless $packet;				# uncoverable branch true
 
 		my @answer = $packet->answer;			# return authoritative answer
 		return $packet if $packet->header->aa && grep $_->name eq $original->qname, @answer;
@@ -145,54 +129,35 @@ sub send {
 
 		my %zone = reverse %auth;
 		foreach my $zone ( keys %zone ) {
-			$self->_diag(";; cache nameservers for $zone\n");
 			my @nsname = grep $auth{$_} eq $zone, keys %auth;
 			my @list = map $glue{$_} ? $glue{$_} : $_, @nsname;
-			@{$res->{cache}->{$zone}} = @list;
+			@{$res->{persistent}->{$zone}} = @list;
 			return $packet if length($zone) > length($domain);
-			push @$nslist, @list;
+			$self->_diag("cache nameservers for $zone");
+			@$nslist = @list;
 		}
 	}
 
+
 	my $query = new Net::DNS::Packet();
 	$query->push( question => $original );
+	$res = bless {%$res}, qw(Net::DNS::Resolver) if $nslist eq $root;
+	$res->udppacketsize(1024);
 	$res->recurse(0);
 
-	my @a = grep ref($_), @$nslist;
-	splice @a, 0, 0, splice( @a, int( rand scalar @a ) );	# cut deck
+	splice @$nslist, 0, 0, splice( @$nslist, int( rand scalar @$nslist ) );	   # cut deck
 
-	foreach (@a) {
-		my @ns = $res->nameservers(@$_);
+	foreach my $ns (@$nslist) {
+		unless ( ref $ns ) {
+			$self->_diag("find missing glue for $ns");
+			$ns = [$res->nameservers($ns)];		# substitute IP list in situ
+		}
 
-		# uncoverable branch true	# depends on external data
-		next unless scalar @ns;
+		$res->nameservers( map @$_, grep ref($_), @$nslist );
 
 		my $reply = $res->send($query);
-
-		# uncoverable branch true	# depends on external data
-		next unless $reply;
-
-		# uncoverable branch true	# depends on external data
-		next unless $reply->header->rcode eq 'NOERROR';
-
-		$self->_callback($reply);
-		return $reply;
-	}
-
-	foreach my $ns ( grep !ref($_), @$nslist ) {
-		$self->_diag(";; find missing glue for $ns\n");
-		$ns = [$res->nameservers($ns)];			# substitute IP list in situ
-
-		# uncoverable branch true	# depends on external data
-		next unless scalar @$ns;
-
-		my $reply = $res->send($query);
-
-		# uncoverable branch true	# depends on external data
-		next unless $reply;
-
-		# uncoverable branch true	# depends on external data
-		next unless $reply->header->rcode eq 'NOERROR';
+		next unless $reply;				# uncoverable branch true
+		next unless $reply->header->rcode eq 'NOERROR'; # uncoverable branch true
 
 		$self->_callback($reply);
 		return $reply;
@@ -240,12 +205,6 @@ sub recursion_callback { &callback; }				# uncoverable pod
 
 ########################################
 
-sub _diag {				## debug output
-	my $self = shift;
-	print @_ if $self->{debug};				# uncoverable branch true
-}
-
-
 {
 	require Net::DNS::ZoneFile;
 
@@ -313,11 +272,11 @@ L<Net::DNS::Resolver>
 
 __DATA__	## DEFAULT HINTS
 
-; <<>> DiG 9.9.4-P2-RedHat-9.9.4-15.P2.fc20 <<>> @a.root-servers.net . -t NS
+; <<>> DiG 9.9.4-P2-RedHat-9.9.4-18.P2.fc20 <<>> @b.root-servers.net . -t NS
 ; (2 servers found)
 ;; global options: +cmd
 ;; Got answer:
-;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 4589
+;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 20118
 ;; flags: qr aa rd; QUERY: 1, ANSWER: 13, AUTHORITY: 0, ADDITIONAL: 25
 ;; WARNING: recursion requested but not available
 
@@ -342,33 +301,28 @@ __DATA__	## DEFAULT HINTS
 .			518400	IN	NS	e.root-servers.net.
 
 ;; ADDITIONAL SECTION:
-c.root-servers.net.	3600000	IN	A	192.33.4.12
-c.root-servers.net.	3600000	IN	AAAA	2001:500:2::c
-k.root-servers.net.	3600000	IN	A	193.0.14.129
-k.root-servers.net.	3600000	IN	AAAA	2001:7fd::1
-l.root-servers.net.	3600000	IN	A	199.7.83.42
-l.root-servers.net.	3600000	IN	AAAA	2001:500:3::42
-j.root-servers.net.	3600000	IN	A	192.58.128.30
-j.root-servers.net.	3600000	IN	AAAA	2001:503:c27::2:30
-b.root-servers.net.	3600000	IN	A	192.228.79.201
-b.root-servers.net.	3600000	IN	AAAA	2001:500:84::b
-g.root-servers.net.	3600000	IN	A	192.112.36.4
-h.root-servers.net.	3600000	IN	A	128.63.2.53
-h.root-servers.net.	3600000	IN	AAAA	2001:500:1::803f:235
-d.root-servers.net.	3600000	IN	A	199.7.91.13
-d.root-servers.net.	3600000	IN	AAAA	2001:500:2d::d
 a.root-servers.net.	3600000	IN	A	198.41.0.4
-a.root-servers.net.	3600000	IN	AAAA	2001:503:ba3e::2:30
-f.root-servers.net.	3600000	IN	A	192.5.5.241
-f.root-servers.net.	3600000	IN	AAAA	2001:500:2f::f
-i.root-servers.net.	3600000	IN	A	192.36.148.17
-i.root-servers.net.	3600000	IN	AAAA	2001:7fe::53
-m.root-servers.net.	3600000	IN	A	202.12.27.33
-m.root-servers.net.	3600000	IN	AAAA	2001:dc3::35
+b.root-servers.net.	3600000	IN	A	192.228.79.201
+c.root-servers.net.	3600000	IN	A	192.33.4.12
+d.root-servers.net.	3600000	IN	A	199.7.91.13
 e.root-servers.net.	3600000	IN	A	192.203.230.10
-
-;; Query time: 29 msec
-;; SERVER: 198.41.0.4#53(198.41.0.4)
-;; WHEN: Mon Aug 11 14:39:19 BST 2014
-;; MSG SIZE  rcvd: 755
+f.root-servers.net.	3600000	IN	A	192.5.5.241
+g.root-servers.net.	3600000	IN	A	192.112.36.4
+h.root-servers.net.	3600000	IN	A	198.97.190.53
+i.root-servers.net.	3600000	IN	A	192.36.148.17
+j.root-servers.net.	3600000	IN	A	192.58.128.30
+k.root-servers.net.	3600000	IN	A	193.0.14.129
+l.root-servers.net.	3600000	IN	A	199.7.83.42
+m.root-servers.net.	3600000	IN	A	202.12.27.33
+a.root-servers.net.	3600000	IN	AAAA	2001:503:ba3e::2:30
+b.root-servers.net.	3600000	IN	AAAA	2001:500:84::b
+c.root-servers.net.	3600000	IN	AAAA	2001:500:2::c
+d.root-servers.net.	3600000	IN	AAAA	2001:500:2d::d
+f.root-servers.net.	3600000	IN	AAAA	2001:500:2f::f
+h.root-servers.net.	3600000	IN	AAAA	2001:500:1::53
+i.root-servers.net.	3600000	IN	AAAA	2001:7fe::53
+j.root-servers.net.	3600000	IN	AAAA	2001:503:c27::2:30
+k.root-servers.net.	3600000	IN	AAAA	2001:7fd::1
+l.root-servers.net.	3600000	IN	AAAA	2001:500:3::42
+m.root-servers.net.	3600000	IN	AAAA	2001:dc3::35
 
